<?php

namespace app\controllers;

use Yii;
use yii\web\Controller;
use yii\web\Response;
use app\models\Product;
use yii\web\Cookie;

class SearchController extends Controller
{
    public function actionIndex(): array|string
    {
        // Инициализация
        $request = Yii::$app->request->post('request') ?? Yii::$app->request->get('q');

        if (Yii::$app->request->post('type') === 'product' || Yii::$app->request->get('type') === 'product') {
            // Поиск по продуктам

            // Инициализация сессии
            $session = Yii::$app->session;
            $session->open();

            // Инициализация ответа
            $response = null;

            // Период пропуска запросов (в секундах)
            $period = 1;

            // Запись времени последнего запроса и вычисление об истечении таймера
            $timer = ($session['last_request'] ?? $session['last_request'] = time() + $period) - time();

            if ($timer > 0) {
                // Ожидание перед повторным запросом при условии что старых запросов нет

                Yii::$app->response->statusCode = 202;

                $return = [
                    'timer' => $timer,
                    'search_line_window' => $this->renderPartial('/loading'),
                    '_csrf' => Yii::$app->request->getCsrfToken()
                ];
            } else {
                // Повторный запрос по истечению ожидания

                // Очистка времени последнего запроса
                unset($session['last_request']);

                //
                // Здесь запись истории запросов (в базе данных)
                //

                if ($response = Product::search($request)) {
                    // Данные найдены

                    // Запись ответа
                    $return = [
                        'search_line_window' => $this->renderPartial('/search/panel', compact('response')),
                        '_csrf' => Yii::$app->request->getCsrfToken()
                    ];

                    if ((int) Yii::$app->request->post('advanced')) {
                        // Полноценный поиск

                        // Запись ответа
                        $return['main'] = $this->renderPartial('/search/index', compact('response'));
                        $return['search_line_window_hide'] = 1;
                        $return['redirect'] = '/search?type=product&q=' . $request;
                    }
                } else {
                    // Данные не найдены

                    Yii::$app->response->statusCode = 404;
                }
            }

            if (Yii::$app->request->isAjax) {
                // AJAX-POST-запрос

                Yii::$app->response->format = Response::FORMAT_JSON;

                return $return ?? [
                    'search_line_window' => $this->renderPartial('/search/panel'),
                    '_csrf' => Yii::$app->request->getCsrfToken()
                ];
            } else {
                // GET-запрос

                return $this->render('/search/index', compact('response', 'timer'));
            }
        }

        if (Yii::$app->user->isGuest) {
            return $this->render('/search/index', ['error_auth' => true]);
        } else {
            return $this->render('/search/index');
        }
    }
}
