<?php

namespace app\models;

use Yii;
use yii\web\IdentityInterface;
use carono\exchange1c\interfaces\PartnerInterface;

class Account extends Document implements IdentityInterface, PartnerInterface
{
    public static function collectionName()
    {
        return 'account';
    }

    public function attributes()
    {
        return array_merge(
            parent::attributes(),
            [
                'mail',
                'pswd',
                'name',
                'simc',
                'sity',
                'comp',
                'taxn',
                'auth'
            ]
        );
    }

    public function rules()
    {
        return array_merge(
            parent::rules(),
            [
                [['mail', 'pswd'], 'required', 'message' => 'Заполните поле'],
                ['mail', 'email'],
                [['mail', 'comp', 'simc'], 'unique', 'message' => '2']
            ]
        );
    }

    public function attributeLabels()
    {
        return array_merge(
            parent::attributeLabels(),
            [
                'mail' => 'Почта',
                'pswd' => 'Пароль',
                'name' => 'Имя',
                'simc' => 'Номер',
                'sity' => 'Город',
                'comp' => 'Компания',
                'taxn' => 'ИНН',
                'auth' => 'Аутентификационный хеш'
            ]
        );
    }

    public function beforeSave($data)
    {
        if (parent::beforeSave($data)) {
            if ($this->isNewRecord) {
                $this->auth = Yii::$app->security->generateRandomString();
            }

            return true;
        }
        return false;
    }

    public function getExportFields1c($context = null)
    {
        return [
            'Ид' => 'id',
            'Наименование' => 'username',
            'ПолноеНаименование' => 'full_name',
            'Фамилия' => 'surname',
            'Имя' => 'name',
        ];
    }

    public function getId(): string
    {
        return $this->_key;
    }

    public function readId(): string
    {
        return self::collectionName() . '/' . $this->getId();
    }

    public function getAuthKey()
    {
        return $this->auth;
    }

    public static function findIdentity($_key)
    {
        return static::findByKey($_key);
    }

    public static function findIdentityByAccessToken($pass, $type = null)
    {
        return static::findOne(['pass' => $pass]);
    }

    public static function findByMail($mail)
    {
        return static::findOne(['mail' => $mail]);
    }

    public static function findByKey($_key)
    {
        return static::findOne(['_key' => $_key]);
    }

    public static function validateMail($mail)
    {
        if (static::findByMail($mail)) {
            // Почта найдена в базе данных

            return false;
        }

        return true;
    }

    public function validatePassword($pswd)
    {
        return Yii::$app->security->validatePassword($pswd, $this->pswd);
    }

    public function validateAuthKey($auth)
    {
        return $this->getAuthKey() === $auth;
    }
}
