<?php

namespace app\models;

use moonland\phpexcel\Excel;
use mirzaev\yii2\arangodb\Query;

class Product extends Document
{
    const SCENARIO_IMPORT = 'import';
    const SCENARIO_WRITE = 'write';

    public $file;
    public $group;

    public static function collectionName(): string
    {
        return 'product';
    }

    public function attributes(): array
    {
        return array_merge(
            parent::attributes(),
            [
                'name',
                'ocid',
                'catn',
                'oemn'
                // 'data',
                // 'cost',
                // 'time'
            ]
        );
    }

    public function rules(): array
    {
        return array_merge(
            parent::rules(),
            [
                [
                    ['name', 'catn'],
                    'required',
                    'message' => 'Заполните поля: {attribute}',
                    'on' => self::SCENARIO_WRITE,
                    'except' => self::SCENARIO_IMPORT
                ],
                [
                    'file',
                    'required',
                    'message' => 'Заполните поля: {attribute}',
                    'on' => self::SCENARIO_IMPORT
                ],
                ['catn', 'string', 'message' => '{attribute} должен быть строкой'],
                // ['oemn', 'integer'], Нужна своя проверка на массив
                [
                    'file',
                    'file',
                    'skipOnEmpty' => false,
                    'extensions' => 'xlsx',
                    'checkExtensionByMimeType' => false,
                    'maxFiles' => 5,
                    'maxSize' => 1024 * 1024 * 30,
                    'wrongExtension' => 'Разрешены только документы в формате: ".xlsx"',
                    'message' => 'Проблема при чтении документа',
                    'on' =>  self::SCENARIO_IMPORT
                ]
            ]
        );
    }

    public function attributeLabels(): array
    {
        return array_merge(
            parent::attributeLabels(),
            [
                'name' => 'Название (name)',
                'ocid' => 'Идентификатор 1C (ocid)',
                'catn' => 'Каталожный номер (catn)',
                'oemn' => 'OEM номера (oemn)',
                // 'data' => 'Данные товара (data)',
                // 'cost' => 'Цены (cost)',
                // 'time' => 'Сроки доставки (time)',
                'file' => 'Документ',
                'group' => 'Группа'
            ]
        );
    }

    public function import()
    {
        // Инициализация массива данных
        $data = [];

        if ($this->validate()) {
            foreach ($this->file as $file) {
                // Перебор файлов

                // Сохранение на диск
                if (!file_exists('../assets/import/excel/')) {
                    mkdir('../assets/import/excel/', 0775, true);
                }
                $file->saveAs($path = '../assets/import/excel/' . $file->baseName . '.' . $file->extension);

                // Проверка файла пройдена

                $data[] = Excel::import($path, [
                    'setFirstRecordAsKeys' => true,
                    'setIndexSheetByName' => true,
                ]);
            }

            foreach ($data[0] as $doc) {
                // Перебор полученных документов

                // Сохранение в базе данных
                $product = new static($doc);

                if ($product->validate()) {
                    // Проверка пройдена

                    // Запись документа
                    $product->save();

                    // Запись группы
                    $group = static::class . 'Group';
                    (new $group())->writeMember($product, $this->group);
                } else {
                    // Проверка не пройдена
                    foreach ($product->errors as $attribute => $error) {
                        $this->addError($attribute, $error);
                    }
                }
            }

            return true;
        }

        return false;
    }

    public static function search(string $text): array
    {
        return (new Query)->limit(10)->search('product_search', ['id' => '_key', 'catn' => 'catn'], ['catn' => $text], 1);
    }

    private static function writeEdgeBetweenGroup(string $from, string $to): bool
    {
        // Инициализация
        $edge = new SupplyEdgeSupplyGroup();

        // Настройка
        $edge->_from = $from;
        $edge->_to = $to;

        // Запись
        return $edge->save();
    }

    private static function writeEdgeBetweenRequisite(string $from, string $to): bool
    {
        // Инициализация
        $edge = new SupplyEdgeRequisite();

        // Настройка
        $edge->_from = $from;
        $edge->_to = $to;

        // Запись
        return $edge->save();
    }

    public static function readById(string $_key): ?Product
    {
        return self::findOne(['_key' => $_key]);
    }

    /**
     * Поиск по каталожному номеру
     */
    public static function readByCatn(string $catn): ?Product
    {
        return self::findOne(['catn' => $catn]);
    }
}
